////////////////////////////////////////////////////////////////////////////////////
//
//			http://www.interestprograms.ru - ,     
//
/////////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "CubeRubika3.h"
#include ".\background.h"


CBackground::CBackground(void)
:  m_ColorBG(D3DCOLOR_XRGB(0, 0, 0))
, m_pTexture_BG(NULL)
, m_pVB_BG(NULL)
, m_IsTextureBG(FALSE)
{
}

CBackground::~CBackground(void)
{
	SAFE_RELEASE(m_pVB_BG);
	SAFE_RELEASE(m_pTexture_BG);
}

///////////////////////////////
HRESULT CBackground::InitVertex(LPDIRECT3DDEVICE9 pd3dDevice)
{
	if(pd3dDevice == NULL) return E_FAIL;

	//   D3DPOOL_MANAGED      
	//   .
	HRESULT hr = pd3dDevice->CreateVertexBuffer( 6*sizeof(VERTEXBG),
												 0, D3DFVF_VERTEXBG,
												 D3DPOOL_MANAGED, &m_pVB_BG, 0 );
	return hr;
}


HRESULT CBackground::Render(LPDIRECT3DDEVICE9 pd3dDevice)
{
	if(pd3dDevice == NULL) return E_FAIL;

	//      .
	if(m_IsTextureBG == TRUE)
	{
		//       
		//     .
		pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

		//    ,
		//       - .
		D3DXMATRIX matWorld;
		D3DXMatrixIdentity(&matWorld);
		pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld);

		pd3dDevice->SetTexture(0, m_pTexture_BG);
		pd3dDevice->SetStreamSource(0, m_pVB_BG, 0, sizeof(VERTEXBG));
		pd3dDevice->SetFVF(D3DFVF_VERTEXBG);
		pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);
	}

	return S_OK;
}

HRESULT CBackground::SetTexture(LPDIRECT3DDEVICE9 pd3dDevice, const CString& filename)
{
	//    ,    .
	SAFE_RELEASE(m_pTexture_BG);

	return D3DXCreateTextureFromFile(pd3dDevice, filename, &m_pTexture_BG);
}

void CBackground::LoadPicture(LPDIRECT3DDEVICE9 pd3dDevice)
{
	CFileDialog open_dlg(TRUE);
	open_dlg.m_pOFN->lpstrTitle = "   ";
	if(open_dlg.DoModal() == IDOK)
	{
		m_StoreFileImage = open_dlg.GetPathName();
		SetTexture(pd3dDevice, m_StoreFileImage);

		//    ,    .
		m_IsTextureBG = TRUE;
	}
}

//    
void CBackground::SetColorBG(void)
{
	CColorDialog colordlg;
	if(colordlg.DoModal() == IDOK)
	{
		COLORREF color = colordlg.GetColor();
		m_ColorBG = D3DCOLOR_XRGB(GetRValue(color), GetGValue(color), GetBValue(color));

		//      ,   .
		m_IsTextureBG = FALSE;
	}
}


//         .
HRESULT CBackground::Restore(float x, float y)
{
	HRESULT hr = S_OK;
	hr = m_pVB_BG->Lock( 0, 0, (void**)&m_pVertices_BG, 0);
	if(hr != S_OK)	return hr;

	//     Z ""   .
	float Z_vFromPt = 100;

	float temp = x*Z_vFromPt;
	x = temp;

	temp = y*Z_vFromPt;
	y = temp;
	
	//     
	D3DCOLOR color = D3DCOLOR_XRGB(255, 255, 255);
	
	//  
	m_pVertices_BG[0].x = -x;
	m_pVertices_BG[0].y = y;
	m_pVertices_BG[0].z = Z_vFromPt;
	m_pVertices_BG[0].color = color;
	m_pVertices_BG[0].tu = 0.0; 
	m_pVertices_BG[0].tv = 0.0;
	
	
	m_pVertices_BG[1].x =  x;
	m_pVertices_BG[1].y =  y;
	m_pVertices_BG[1].z = Z_vFromPt;
	m_pVertices_BG[1].color = color;
	m_pVertices_BG[1].tu = 1.0;
	m_pVertices_BG[1].tv = 0.0;
	
	m_pVertices_BG[2].x = -x;
	m_pVertices_BG[2].y =  -y;
	m_pVertices_BG[2].z = Z_vFromPt;
	m_pVertices_BG[2].color = color;
	m_pVertices_BG[2].tu = 0.0;
	m_pVertices_BG[2].tv = 1.0;

	//  
	m_pVertices_BG[3].x = x;
	m_pVertices_BG[3].y = y;
	m_pVertices_BG[3].z = Z_vFromPt;
	m_pVertices_BG[3].color = color;
	m_pVertices_BG[3].tu = 1.0; 
	m_pVertices_BG[3].tv = 0.0;
	
	
	m_pVertices_BG[4].x =  x;
	m_pVertices_BG[4].y = -y;
	m_pVertices_BG[4].z = Z_vFromPt;
	m_pVertices_BG[4].color = color;
	m_pVertices_BG[4].tu = 1.0;
	m_pVertices_BG[4].tv = 1.0;
	
	m_pVertices_BG[5].x = -x; 
	m_pVertices_BG[5].y = -y;
	m_pVertices_BG[5].z = Z_vFromPt;
	m_pVertices_BG[5].color = color;
	m_pVertices_BG[5].tu = 0.0;
	m_pVertices_BG[5].tv = 1.0;



	return m_pVB_BG->Unlock();
}

void CBackground::DataToRegistry(HKEY hKey)
{
	GlobalStringToRegKey(hKey, "ImageBackground", m_StoreFileImage.GetBuffer());
	GlobalNumberToRegKey(hKey, "TextureBG", (DWORD)m_IsTextureBG);
	GlobalNumberToRegKey(hKey, "ColorBG", m_ColorBG);
	
}

void CBackground::DataFromRegistry(HKEY hKey, LPDIRECT3DDEVICE9 pd3dDevice)
{
	GlobalStringFromRegKey(hKey, "ImageBackground", m_StoreFileImage.GetBuffer(512), 512, "1.jpg"); 
	
	//      ,
	//       .
	m_StoreFileImage.ReleaseBuffer();
	SetTexture(pd3dDevice, m_StoreFileImage);

	GlobalNumberFromRegKey(hKey, "ColorBG", &m_ColorBG, D3DCOLOR_XRGB(0, 0, 0));
	GlobalNumberFromRegKey(hKey, "TextureBG", (DWORD*)&m_IsTextureBG, FALSE);
}

